@@ -15,17 +15,19 @@
 
 import collections
 import gc
+import io
+import ipaddress
 import json
 import platform
 import socket
 import sys
 import traceback
 
+from debtcollector import removals
 import jinja2
 from oslo_utils import reflection
 from oslo_utils import strutils
 from oslo_utils import timeutils
-import six
 import stevedore
 import webob.dec
 import webob.exc
@@ -51,11 +53,10 @@ def _expand_template(contents, params):
 
 
 class Healthcheck(base.ConfigurableMiddleware):
-    """Healthcheck middleware used for monitoring.
+    """Healthcheck application used for monitoring.
 
-    If the path is ``/healthcheck``, it will respond 200 with "OK" as
-    the body. Or a 503 with the reason as the body if one of the backends
-    reports an application issue.
+    It will respond 200 with "OK" as the body. Or a 503 with the reason as the
+    body if one of the backends reports an application issue.
 
     This is useful for the following reasons:
 
@@ -65,6 +66,12 @@ class Healthcheck(base.ConfigurableMiddleware):
       the activity of the server.
     * *(and more)*
 
+    .. note::
+
+        This middleware indicates that the API is accessible but it does
+        indicate that it is necessarily functional or that any other API
+        request will actually work.
+
     Example requests/responses (**not** detailed mode)::
 
       $ curl -i -X HEAD "http://0.0.0.0:8775/healthcheck"
@@ -239,9 +246,8 @@ class Healthcheck(base.ConfigurableMiddleware):
 
     .. code-block:: ini
 
-        [filter:healthcheck]
-        paste.filter_factory = oslo_middleware:Healthcheck.factory
-        path = /healthcheck
+        [app:healthcheck]
+        use = egg:oslo.middleware:healthcheck
         backends = disable_by_file
         disable_by_file_path = /var/run/nova/healthcheck_disable
 
@@ -253,21 +259,29 @@ class Healthcheck(base.ConfigurableMiddleware):
 
     .. code-block:: ini
 
-        [pipeline:public_api]
-        pipeline = healthcheck_public sizelimit [...] public_service
+        [composite:public_api]
+        use = egg:Paste#urlmap
+        / = public_api_pipeline
+        /healthcheck = healthcheck_public
 
-        [pipeline:admin_api]
-        pipeline = healthcheck_admin sizelimit [...] admin_service
+        [composite:admin_api]
+        use = egg:Paste#urlmap
+        / = admin_api_pipeline
+        /healthcheck = healthcheck_admin
 
-        [filter:healthcheck_public]
-        paste.filter_factory = oslo_middleware:Healthcheck.factory
-        path = /healthcheck_public
+        [pipeline:public_api_pipeline]
+        pipeline = sizelimit [...] public_service
+
+        [pipeline:admin_api_pipeline]
+        pipeline = sizelimit [...] admin_service
+
+        [app:healthcheck_public]
+        use = egg:oslo.middleware:healthcheck
         backends = disable_by_file
         disable_by_file_path = /var/run/nova/healthcheck_public_disable
 
         [filter:healthcheck_admin]
-        paste.filter_factory = oslo_middleware:Healthcheck.factory
-        path = /healthcheck_admin
+        use = egg:oslo.middleware:healthcheck
         backends = disable_by_file
         disable_by_file_path = /var/run/nova/healthcheck_admin_disable
     """
@@ -378,6 +392,9 @@ Reason
                                      group='healthcheck')
         self._path = self._conf_get('path')
         self._show_details = self._conf_get('detailed')
+        self._source_ranges = [
+            ipaddress.ip_network(r)
+            for r in self._conf_get('allowed_source_ranges')]
         self._backends = stevedore.NamedExtensionManager(
             self.NAMESPACE, self._conf_get('backends'),
             name_order=True, invoke_on_load=True,
@@ -388,15 +405,40 @@ Reason
             ('text/html', self._make_html_response),
             ('application/json', self._make_json_response),
         ])
-        self._accept_order = tuple(six.iterkeys(self._accept_to_functor))
+        self._accept_order = tuple(self._accept_to_functor)
         # When no accept type matches instead of returning 406 we will
         # always return text/plain (because sending an error from this
         # middleware actually can cause issues).
         self._default_accept = 'text/plain'
+        self._ignore_path = False
 
     def _conf_get(self, key, group='healthcheck'):
         return super(Healthcheck, self)._conf_get(key, group=group)
 
+    @removals.remove(
+        message="The healthcheck middleware must now be configured as "
+        "an application, not as a filter")
+    @classmethod
+    def factory(cls, global_conf, **local_conf):
+        return super(Healthcheck, cls).factory(global_conf, **local_conf)
+
+    @classmethod
+    def app_factory(cls, global_conf, **local_conf):
+        """Factory method for paste.deploy.
+
+        :param global_conf: dict of options for all middlewares
+                            (usually the [DEFAULT] section of the paste deploy
+                            configuration file)
+        :param local_conf: options dedicated to this middleware
+                           (usually the option defined in the middleware
+                           section of the paste deploy configuration file)
+        """
+        conf = global_conf.copy() if global_conf else {}
+        conf.update(local_conf)
+        o = cls(application=None, conf=conf)
+        o._ignore_path = True
+        return o
+
     @staticmethod
     def _get_threadstacks():
         threadstacks = []
@@ -405,8 +447,8 @@ Reason
         except AttributeError:
             pass
         else:
-            buf = six.StringIO()
-            for stack in six.itervalues(active_frames):
+            buf = io.StringIO()
+            for stack in active_frames.values():
                 traceback.print_stack(stack, file=buf)
                 threadstacks.append(buf.getvalue())
                 buf.seek(0)
@@ -417,7 +459,7 @@ Reason
     def _get_greenstacks():
         greenstacks = []
         if greenlet is not None:
-            buf = six.StringIO()
+            buf = io.StringIO()
             for gt in _find_objects(greenlet.greenlet):
                 traceback.print_stack(gt.gr_frame, file=buf)
                 greenstacks.append(buf.getvalue())
@@ -510,8 +552,19 @@ Reason
 
     @webob.dec.wsgify
     def process_request(self, req):
-        if req.path != self._path:
+        if not self._ignore_path and req.path != self._path:
             return None
+
+        if self._source_ranges:
+            remote_addr = ipaddress.ip_address(req.remote_addr)
+            for r in self._source_ranges:
+                if r.version == remote_addr.version and remote_addr in r:
+                    break
+            else:
+                # Because source ip is not included in allowed ranges, ignore
+                # the request in this middleware.
+                return None
+
         results = [ext.obj.healthcheck(req.server_port)
                    for ext in self._backends]
         healthy = self._are_results_healthy(results)
@@ -520,10 +573,13 @@ Reason
             status = self.HEAD_HEALTHY_TO_STATUS_CODES[healthy]
         else:
             status = self.HEALTHY_TO_STATUS_CODES[healthy]
-            accept_type = req.accept.best_match(self._accept_order)
-            if not accept_type:
+            try:
+                offers = req.accept.acceptable_offers(self._accept_order)
+                accept_type = offers[0][0]
+            except IndexError:
                 accept_type = self._default_accept
             functor = self._accept_to_functor[accept_type]
         body, content_type = functor(results, healthy)
         return webob.response.Response(status=status, body=body,
+                                       charset='UTF-8',
                                        content_type=content_type)
